"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const xlsx_1 = __importDefault(require("xlsx"));
const utils_1 = require("./utils");
class ExcelFile {
    constructor(name, data, options) {
        this.name = name;
        this.data = data;
        this.options = options;
        /**
         * Return fileName with date time at the end of the name
         */
        this.addDateTimeToName = (name) => {
            const { year, month, day, hours, minutes, seconds } = utils_1.getDateTimeStrings(new Date());
            const dateString = `${year}-${month}-${day}`;
            const timeString = `${hours}_${minutes}_${seconds}`;
            return `${name}_${dateString}T${timeString}`;
        };
        this.validateName(name);
        this.validateData(data);
        this.name = (options === null || options === void 0 ? void 0 : options.isNameHasDateTime)
            ? this.addDateTimeToName(name)
            : name;
        this.data = this.formatData(data);
        this.wb = this.createWorkBook(this.data);
    }
    /**
     * Check name validation
     * It throws error when name is null or empty
     * @param name
     */
    validateName(name) {
        if (name === null || name.length === 0) {
            throw Error('Invalid file name provided');
        }
        return;
    }
    /**
     * Check data validation
     * It throws error when data is empty or has different row length
     * @param data
     */
    validateData(data) {
        if (data.length === 0 || data[0].length === 0) {
            throw Error('Invalid data provided');
        }
        if (data.find((row) => row.length !== data[0].length)) {
            throw Error('All rows should be a same length');
        }
        return;
    }
    /**
     * Return formatted data
     * @param data
     */
    formatData(data) {
        return data.map((row) => row.map((cell) => {
            if (cell === null) {
                return '';
            }
            if (typeof cell === 'string') {
                return cell;
            }
            if (typeof cell === 'number') {
                return cell.toString();
            }
            if (typeof cell === 'boolean') {
                return cell.toString().toUpperCase();
            }
            if (utils_1.isDate(cell)) {
                return utils_1.formatDate(cell);
            }
            if (typeof cell === 'object') {
                return JSON.stringify(cell);
            }
            return '';
        }));
    }
    /**
     * Create workSheet from provided data
     */
    createWorkSheet(data) {
        return xlsx_1.default.utils.aoa_to_sheet(data);
    }
    /**
     * Create workBook with only one sheet.
     * Appended sheet will be named 'Sheet1' for default.
     */
    createWorkBook(data) {
        const workBook = xlsx_1.default.utils.book_new();
        const workSheet = this.createWorkSheet(data);
        xlsx_1.default.utils.book_append_sheet(workBook, workSheet);
        return workBook;
    }
    /**
     * Format file full name by joinning name and extension
     */
    formatFullName(name, extension) {
        return `${name}.${extension}`;
    }
    /**
     * Download xlsx or csv file
     * @param extension file extension, default is 'xlsx'
     */
    download(extension = 'xlsx') {
        const fileFullName = this.formatFullName(this.name, extension);
        xlsx_1.default.writeFile(this.wb, fileFullName);
    }
}
exports.default = ExcelFile;
